package cn.com.qmth.scancloud.controller.admin;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.retrofit.utils.SignatureInfo;
import com.qmth.boot.core.retrofit.utils.UploadFile;
import com.qmth.boot.tools.signature.SignatureType;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.client.MarkingcloudApiClient;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.entity.SubjectEntity;
import cn.com.qmth.scancloud.entity.SystemConfigEntity;
import cn.com.qmth.scancloud.enums.FormatType;
import cn.com.qmth.scancloud.enums.SystemMode;
import cn.com.qmth.scancloud.model.FileUploadResponse;
import cn.com.qmth.scancloud.service.SubjectService;
import cn.com.qmth.scancloud.service.SystemConfigService;
import cn.com.qmth.scancloud.vo.UpdateTimeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "试卷答案文件上传接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/file")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class FileController extends BaseController {

    private static final Logger log = LoggerFactory.getLogger(FileController.class);

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SystemConfigService systemConfigService;

    @Autowired
    private MarkingcloudApiClient markingcloudApiClient;

    @ApiOperation(value = "答案上传接口")
    @RequestMapping(value = "/answer/upload", method = RequestMethod.POST)
    public UpdateTimeVo answerUpload(@RequestParam Long examId, @RequestParam String subjectCode,
            @RequestParam String md5, @RequestParam MultipartFile file) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (!SystemMode.MARKING_CLOUD.equals(conf.getSystemMode())) {
            throw new ParameterException("K12模式不允许创建通卡");
        }
        SubjectEntity sub = subjectService.findByExamIdAndCode(examId, subjectCode);
        if (sub == null) {
            throw new ParameterException("未找到科目,请先同步或创建考生获取科目");
        }
        User user = getAccessUser();
        SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                user.getMarkingCloudToken());
        try {
            FileUploadResponse res = markingcloudApiClient.answerFileUpload(signatureInfo.toString(), examId,
                    subjectCode, FormatType.PDF.toString(), md5, UploadFile.build("file", "", file.getBytes()));
            if (res == null || !res.getSuccess()) {
                throw new StatusException("答案上传出错，examId=" + examId + ", subjectCode=" + subjectCode);
            }
        } catch (Exception e) {
            log.error("答案上传出错，examId=" + examId + ", subjectCode=" + subjectCode, e);
        }
        return result(System.currentTimeMillis());
    }

    @ApiOperation(value = "试卷上传接口")
    @RequestMapping(value = "/paper/upload", method = RequestMethod.POST)
    public UpdateTimeVo paperUpload(@RequestParam Long examId, @RequestParam String subjectCode,
            @RequestParam String md5, @RequestParam MultipartFile file) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (!SystemMode.MARKING_CLOUD.equals(conf.getSystemMode())) {
            throw new ParameterException("K12模式不允许创建通卡");
        }
        SubjectEntity sub = subjectService.findByExamIdAndCode(examId, subjectCode);
        if (sub == null) {
            throw new ParameterException("未找到科目,请先同步或创建考生获取科目");
        }
        User user = getAccessUser();
        SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                user.getMarkingCloudToken());
        try {
            FileUploadResponse res = markingcloudApiClient.paperFileUpload(signatureInfo.toString(), examId,
                    subjectCode, FormatType.PDF.toString(), md5, UploadFile.build("file", "", file.getBytes()));
            if (res == null || !res.getSuccess()) {
                throw new StatusException("试卷上传出错，examId=" + examId + ", subjectCode=" + subjectCode);
            }
        } catch (Exception e) {
            log.error("试卷上传出错，examId=" + examId + ", subjectCode=" + subjectCode, e);
        }
        return result(System.currentTimeMillis());
    }

}
