package cn.com.qmth.scancloud.multithread.consumer;

import cn.com.qmth.scancloud.entity.StudentEntity;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.enums.UploadStatus;
import cn.com.qmth.scancloud.multithread.BatchConsumer;
import cn.com.qmth.scancloud.service.StudentService;
import cn.com.qmth.scancloud.service.ToolExportService;
import cn.com.qmth.scancloud.vo.StudentUploadVo;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Scope("prototype")
@Service
public class PictureCopyConsumer extends BatchConsumer<StudentUploadVo> {

    private static final Logger log = LoggerFactory.getLogger(PictureCopyConsumer.class);

    @Autowired
    private StudentService studentService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private ToolExportService toolExportService;

    @Transactional
    @Override
    public void consume(StudentUploadVo s) {
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + s.getId()).writeLock().lock();
        StudentEntity se = studentService.getById(s.getId());
        try {
            if (UploadStatus.WAITING_UPLOAD.equals(se.getFileUploadStatus())) {
                toolExportService.studentFileCopy(se.getId(),s.getImageTransferMode());
                studentService.updateFileUploadStatus(se.getId(), UploadStatus.UPLOADED);
            }
        } catch (Exception e) {
            studentService.updateFileUploadStatus(se.getId(), UploadStatus.ERROR);
            log.error("转存文件出错.studentId:" + se.getId(), e);
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + s.getId()).writeLock().unlock();
        }
    }

}
