package cn.com.qmth.scancloud.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancloud.dao.ScannerDao;
import cn.com.qmth.scancloud.entity.AnswerCardEntity;
import cn.com.qmth.scancloud.entity.ScannerEntity;
import cn.com.qmth.scancloud.entity.SystemConfigEntity;
import cn.com.qmth.scancloud.service.AnswerCardService;
import cn.com.qmth.scancloud.service.ScannerService;
import cn.com.qmth.scancloud.service.SessionService;
import cn.com.qmth.scancloud.service.SubjectService;
import cn.com.qmth.scancloud.vo.scannerinfo.Control;
import cn.com.qmth.scancloud.vo.scannerinfo.ScannerInfo;
import cn.com.qmth.scancloud.vo.scannerinfo.ScannerInfoCard;
import cn.com.qmth.scancloud.vo.scannerinfo.ScannerInfoVo;

@Service
public class ScannerServiceImpl extends ServiceImpl<ScannerDao, ScannerEntity> implements ScannerService {
	
	@Autowired
	private SystemConfigServiceImpl systemConfigService;
	
	@Autowired
	private SessionService sessionService;
	
	@Autowired
	private AnswerCardService answerCardService;
	
	@Autowired
	private SubjectService subjectService;

	@Transactional
	@Override
	public void onLogin(String device) {
		if(StringUtils.isBlank(device)) {
			throw new ParameterException("设备信息不能为空");
		}
		ScannerEntity e=new ScannerEntity();
		e.setDevice(device);
		e.setLastLoginTime(new Date().getTime());
		this.saveOrUpdate(e);
	}

	@Override
	public ScannerInfoVo scannerInfo(Long examId) {
		ScannerInfoVo vo=new ScannerInfoVo();
		vo.setControl(new Control());
		SystemConfigEntity config=systemConfigService.findConfig();
		if(config==null) {
			vo.getControl().setEnableLogin(true);
		}else {
			vo.getControl().setEnableLogin(config.getScannerEnableLogin()==null?true:config.getScannerEnableLogin());
			vo.getControl().setPassword(config.getScannerPassword());
		}
        List<ScannerInfo> infos=baseMapper.getInfoByExam(examId);
        if(infos==null) {
        	infos=new ArrayList<>();
        }
        vo.setList(infos);
        for(ScannerInfo s:infos) {
        	if(s.getAnswerCardNumber()!=null) {
        		AnswerCardEntity card=answerCardService.findByExamAndNumber(examId, s.getAnswerCardNumber());
        		ScannerInfoCard c=new ScannerInfoCard();
        		c.setNumber(card.getNumber());
        		c.setRemark(card.getRemark());
        		c.setSubjectCode(card.getSubjectCode());
        		c.setSubjectName(subjectService.findByExamIdAndCode(examId, card.getSubjectCode()).getName());
        		s.setAnswerCard(c);
        	}
        	s.setOnline(sessionService.checkScannerOnline(s.getDevice()));
        }
		return vo;
	}

}
