package cn.com.qmth.scancloud.service.impl;

import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.scancloud.bean.SchoolSession;
import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.config.SysProperty;
import cn.com.qmth.scancloud.enums.Role;
import cn.com.qmth.scancloud.enums.SystemMode;
import cn.com.qmth.scancloud.service.SessionService;
import cn.com.qmth.scancloud.service.SystemConfigService;

@Service
public class SessionServiceImpl implements SessionService {

    private final static Map<String, User> userSessionData = new ConcurrentHashMap<>();

    private final static Map<Long, SchoolSession> schoolSessionData = new ConcurrentHashMap<>();

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private SystemConfigService systemConfigService;

    @Override
    public Long updateUserSession(User user) {
        if (user.getSessionId() == null) {
            return null;
        }
        user.setActiveTime(new Date().getTime());
        userSessionData.put(user.getSessionId(), user);
        return user.getActiveTime();
    }

    @Override
    public void clearTimeOutUser() {
        Date d = new Date();
        Long now = d.getTime();
        for (String k : userSessionData.keySet()) {
            if (now - userSessionData.get(k).getActiveTime() > getTimeout()) {
                userSessionData.remove(k);
            }
        }
    }

    @Override
    public void userLogout(User user) {
        userSessionData.remove(user.getSessionId());
    }

    @Override
    public void userLogin(User user) {
        updateUserSession(user);
        if (SystemMode.MARKING_CLOUD.equals(systemConfigService.findConfig().getSystemMode())) {
            if (Role.SCAN_ADMIN.equals(user.getRole()) || Role.SCHOOL_ADMIN.equals(user.getRole())) {
                updateSchoolSession(user);
            }
        }
    }

    private void updateSchoolSession(User user) {
        SchoolSession s = schoolSessionData.get(user.getSchoolId());
        if (s == null) {
            schoolSessionData.put(user.getSchoolId(), SchoolSession.of(user));
        } else if (Role.SCAN_ADMIN.equals(s.getRole()) || Role.SCHOOL_ADMIN.equals(user.getRole())) {
            schoolSessionData.put(user.getSchoolId(), SchoolSession.of(user));
        }
    }

    @Override
    public void removeSchoolSession(Long schoolId) {
        schoolSessionData.remove(schoolId);
    }

    @Override
    public Collection<SchoolSession> getSchoolSessions() {
        return schoolSessionData.values();
    }

    @Override
    public SchoolSession getSchoolSession(Long schoolId) {
        return schoolSessionData.get(schoolId);
    }

    @Override
    public User getSessionUser(String sessionId) {
        return userSessionData.get(sessionId);
    }

    @Override
    public int getOnlineCount() {
        return userSessionData.size();
    }

    @Override
    public int getOnlineScannerCount() {
        int count = 0;
        for (User user : userSessionData.values()) {
            if (Role.SCANNER.equals(user.getRole())) {
                count++;
            }
        }
        return count;
    }

    @Override
    public boolean checkScannerOnline(String device) {
        String sessionId = Role.SCANNER.name() + "_" + device;
        return userSessionData.containsKey(sessionId);
    }

    private long getTimeout() {
        return (sysProperty.getHeartbeatTimeout() == null ? 600 : sysProperty.getHeartbeatTimeout()) * 1000L;
    }

    @Override
    public int getOnlineAuditorCount() {
        int count = 0;
        for (User user : userSessionData.values()) {
            if (Role.AUDITOR.equals(user.getRole())) {
                count++;
            }
        }
        return count;
    }

    @Override
    public boolean checkAuditorOnline(String loginName) {
        String sessionId = Role.AUDITOR.name() + "_" + loginName;
        return userSessionData.containsKey(sessionId);
    }

}
