package cn.com.qmth.scancloud.support;

import java.util.HashMap;
import java.util.Map;

public class TaskLockUtil {

    // 识别对照任务并发处理互斥锁
    private static final Map<String, TaskLock> omrTaskMap = new HashMap<>();
    
    // 实时审核任务并发处理互斥锁
    private static final Map<String, TaskLock> verifyTaskMap = new HashMap<>();
    
    // 图片检查任务并发处理互斥锁
    private static final Map<String, TaskLock> checkTaskMap = new HashMap<>();

    public static void clearTimeoutOmrTask(long timeoutMinute) {
        for (TaskLock taskLock : omrTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }
    
    public static void clearTimeoutVerifyTask(long timeoutMinute) {
        for (TaskLock taskLock : verifyTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }
    public static void clearTimeoutCheckTask(long timeoutMinute) {
        for (TaskLock taskLock : checkTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }

    public static TaskLock getOmrTask(String key) {
        TaskLock taskLock = omrTaskMap.get(key);
        if (taskLock == null) {
            synchronized (omrTaskMap) {
                taskLock = omrTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }
    
    public static TaskLock getVerifyTask(String key) {
        TaskLock taskLock = verifyTaskMap.get(key);
        if (taskLock == null) {
            synchronized (verifyTaskMap) {
                taskLock = verifyTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }
    
    public static TaskLock getCheckTask(String key) {
        TaskLock taskLock = checkTaskMap.get(key);
        if (taskLock == null) {
            synchronized (checkTaskMap) {
                taskLock = checkTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }

}
